<?php

if ( ! defined( 'ABSPATH' ) ) exit;

use WordPress\ORM\AbandonedCart;
use WordPress\ORM\RestoredCart;

/**
 * @class CX_Admin_Controller_Dashboard
 * @since 2.8
 */
class CX_Admin_Controller_Dashboard extends CX_Admin_Controller_Abstract {

	/** @var array */
	private static $widgets;

	/** @var array */
	private static $carts_report;

	private static $payment_amount;

	private static $free_trial_amount;


	static function output() {

		wp_enqueue_script( 'chatx-ai-dashboard' );

		self::maybe_set_date_cookie();

		$widgets = self::get_widgets();
		$date_arg = self::get_date_arg();
		$date_range = self::get_date_range();
		$date_tabs = [
			'30days' => __( '30 days', 'automatewoo' ),
			'14days' => __( '14 days', 'automatewoo' ),
			'7days' => __( '7 days', 'automatewoo' )
		];

		foreach ( $widgets as $i => $widget ) {
			$widget->set_date_range( $date_range['from'], $date_range['to'] );
			if ( ! $widget->display ) {
				unset( $widgets[$i] );
			}
		}

		$baseUrl = get_home_url();

		if(substr($baseUrl, -1) == '/') {
		    $baseUrl = substr($baseUrl, 0, -1);
		}

		$queryArray = [
		    'api_type' => 'wp-woocommerce',
		    'api_url' => rest_url('/chatxai-api/v1', 'json')
		];

    	$register_url = CHATX_BAESURL . '/plugin/auth?'.http_build_query($queryArray);

		Chatx_Ai_Admin::get_view( 'page-dashboard', [
			'widgets' => $widgets,
			'date_text' => $date_tabs[$date_arg],
			'date_current' => self::get_date_arg(),
			'date_tabs' => $date_tabs,
			'register_url' => $register_url,
			'is_registered' => CX()->is_registered()
		]);
	}

	/**
	 * @return Dashboard_Widget[]
	 */
	static function get_widgets() {

		if ( ! isset( self::$widgets ) ) {

			$path = CX()->path( '/admin/dashboard-widgets/' );

			$includes = [];

			if ( Chatx_Ai_Options::read('chatxai_enable_carts_tracking') == "yes" ) {
				$includes[] = $path . 'chart-abandoned-carts.php';
				$includes[] = $path . 'chart-recovered-carts.php';
			}

			$recovered_carts_amount = self::get_local_reports('total_recovered_amount');
			$free_trial_amount = self::get_free_trial_amount();

			if ( $free_trial_amount > 0 ) {
				$includes[] = $path . 'free-trial.php';
			} else {
				$includes[] = $path . 'payments.php';
			}

			$includes = apply_filters( 'chatxai/dashboard/widgets', $includes );

			include_once $path . 'abstract.php';
			include_once $path . 'chart-abstract.php';

			$self = new self();

			foreach ( $includes as $include ) {
				/** @var Dashboard_Widget $class */
				$class = include_once $include;
				$class->controller = $self;
				self::$widgets[ $class->id ] = $class;
			}
		}

		return self::$widgets;
	}


	/**
	 * @return string
	 */
	static function get_date_arg() {

		$cookie_name = 'chatxai_dashboard_date';

		if ( ! cx_request( 'date' ) && isset( $_COOKIE[ $cookie_name ] ) ) {
			return CXLibrary\Clean::string( $_COOKIE[ $cookie_name ] );
		}

		if ( cx_request( 'date' ) ) {
			$date = CXLibrary\Clean::string( cx_request( 'date' ) );
			return $date;
		}

		return '30days';
	}


	static function maybe_set_date_cookie() {
		if ( cx_request( 'date' ) ) {
			$date = CXLibrary\Clean::string( cx_request( 'date' ) );
			if ( ! headers_sent() ) wc_setcookie( 'chatxai_dashboard_date', $date, time() + MONTH_IN_SECONDS * 2 );
		}
	}


	/**
	 * @return array
	 */
	static function get_date_range() {

		$range = self::get_date_arg();

		$from = new \DateTime();
		$to = new \DateTime();

		switch ( $range ) {
			case '14days':
				$from->modify( "-14 days" );
				break;
			case '7days':
				$from->modify( "-7 days" );
				break;
			case '30days':
				$from->modify( "-30 days" );
				break;
		}

		return [
			'from' => $from,
			'to' => $to
		];
	}


	static function get_local_reports( $type = 'abandoned' ) {

		switch ($type) {
			case 'total_recovered_amount':
				$total_recovered_amount = 0;

				if (Chatx_Ai_Options::read('chatxai_enable_carts_tracking') == "yes") {
					$query = RestoredCart::query();
					$res = $query->find();

					$total_recovered_amount = 0;

					foreach ( $res as $cart ) {
						$total_recovered_amount += $cart->total;
					}
				}

				return $total_recovered_amount;
				break;
			case 'all_recovered':
				$query = RestoredCart::query();
				$res = $query->find();

				return $res;
				break;
			case 'recovered':
				$date = self::get_date_range();

				$query = RestoredCart::query();
				$query->where_gte('created', $date['from']->format('Y-m-d H:i:s'));
				$query->where_lte('created', $date['to']->format('Y-m-d H:i:s'));
				$res = $query->find();

				return $res;
				break;

			default:
				$date = self::get_date_range();

				$query = AbandonedCart::query();
				$query->where_gte('last_modified', $date['from']->format('Y-m-d H:i:s'));
				$query->where_lte('last_modified', $date['to']->format('Y-m-d H:i:s'));
				$query->where_not('sent', NULL);
				$res = $query->find();

				return $res;
				break;
		}

		return false;

	}


	static function get_reports(DateTime $start, DateTime $end) {

		if ( false === ( $rdata = get_transient( 'chatxai_server_reports' ) ) ) {
	    	// It wasn't there, so regenerate the data and save the transient
	     	$url = CHATX_BAESURL . '/plugin/reports';

		    $data = [
		        "at" => Chatx_Ai_Options::read('chatxai_api_token'),
		        "ak" => Chatx_Ai_Options::read('chatxai_api_key'),
		        "start" => $start->format('d-m-Y'),
		        "end" => $end->format('d-m-Y'),
		        "currency" => get_woocommerce_currency()
		    ];

		    $options = array(
		        'http' => array(
		            'header'  => "Content-type: application/x-www-form-urlencoded\r\n",
		            'method'  => 'POST',
		            'content' => http_build_query($data, true)
		        )
		    );

		    $result = null;

		    try {
		        $context  = stream_context_create($options);
		        $result = file_get_contents($url, false, $context);
		    } catch (Exception $e) { }

		    $rdata = null;

		    if($result){
		        $rdata = json_decode($result);
		    }

	     	set_transient( 'chatxai_server_reports', $rdata, 10);
		}

	    return $rdata;
	}


	/**
	 * @return Abandoned & Restored Carts[]
	 */
	static function get_carts_report() {
		if ( ! isset( self::$carts_report ) && CX()->is_registered() ) {

			$date = self::get_date_range();

			$reports = self::get_reports( $date['from'], $date['to'] );

			if($reports && $reports->success) {
				self::$carts_report = $reports->query;
			}
		}

		return self::$carts_report;
	}

	/**
	 * @return Payment Amount
	 */
	static function get_payment_amount() {
		if ( ! isset( self::$payment_amount ) && CX()->is_registered() ) {

			$date = self::get_date_range();

			$reports = self::get_reports( $date['from'], $date['to'] );

			if($reports && $reports->success) {
				self::$payment_amount = $reports->totals->fee_payable;
			}
		}

		return self::$payment_amount;
	}

	/**
	 * @return Free Trial Amount
	 */
	static function get_free_trial_amount() {
		if ( ! isset( self::$free_trial_amount ) && CX()->is_registered() ) {

			$date = self::get_date_range();

			$reports = self::get_reports( $date['from'], $date['to'] );

			if($reports && $reports->success) {
				self::$free_trial_amount = $reports->free_trial_amount;
			}
		}

		return self::$free_trial_amount;
	}

}
