<?php

/**
 * Register all REST API func for the plugin
 *
 * @package    Chatx_Ai
 * @subpackage Chatx_Ai/includes
 * @author     Chatx.ai <contact@chatx.ai>
 */

use CXLibrary\Notices;

class Chatx_Ai_Rest_API
{

    private $plugin_data = null;
    private $apiKey;
    private $data;
    private $success;
    private $message;
    private $version;
    private $notices = [];

    public function __construct($plugin_data)
    {
        $this->version = isset($plugin_data['Version']) ? $plugin_data['Version'] : '0.0.0';
        $this->data    = [];
        $this->success = false;
        $this->message = 'Unknown message';
    }

    public function init_API()
    {
        $this->apiKey = Chatx_Ai_Options::read('chatxai_api_key');

        add_action('rest_api_init', function () {

            // Get user info by user_ref
            register_rest_route('chatxai-api/v1', '/user/session_key/(?P<session_key>[a-zA-Z0-9=]+)', [
                'methods'  => 'GET',
                'callback' => [$this, 'getUserBySessionKey'],
            ]);

            // Get user info by user_ref
            register_rest_route('chatxai-api/v1', '/user/id/(?P<id>\d+)', [
                'methods'  => 'GET',
                'callback' => [$this, 'getUserById'],
            ]);

            // Get product by id
            register_rest_route('chatxai-api/v1', '/products/(?P<id>\d+)', [
                'methods'  => 'GET',
                'callback' => [$this, 'getProductByID'],
            ]);

            // Create coupon
            register_rest_route('chatxai-api/v1', '/coupons', [
                'methods'  => 'POST',
                'callback' => [$this, 'createCoupon'],
            ]);

            // Create notification
            register_rest_route('chatxai-api/v1', '/notifications', [
                'methods'  => 'POST',
                'callback' => [$this, 'createNotification'],
            ]);

            // Change account status
            register_rest_route('chatxai-api/v1', '/status', [
                'methods'  => 'POST',
                'callback' => [$this, 'changeAccountStatus'],
            ]);

            // Change checkboxes templates
            register_rest_route('chatxai-api/v1', '/settings/checkbox/templates', [
                'methods'  => 'POST',
                'callback' => [$this, 'setCheckboxTemplates'],
            ]);

            // Get crons info
            register_rest_route('chatxai-api/v1', '/settings/crons/data', [
                'methods'  => 'GET',
                'callback' => [$this, 'getCronsData'],
            ]);

            // Reschedule crons
            register_rest_route('chatxai-api/v1', '/settings/crons/reschedule', [
                'methods'  => 'POST',
                'callback' => [$this, 'rescheduleCrons'],
            ]);

            // Get configurations
            register_rest_route('chatxai-api/v1', '/configuration', [
                'methods'  => 'GET',
                'callback' => [$this, 'getWebsiteConfiguration'],
            ]);

            // Get sales reports
            register_rest_route('chatxai-api/v1', '/reports/sales', [
                'methods'  => 'GET',
                'callback' => [$this, 'getSalesReports'],
            ]);

            // Create coupon
            register_rest_route('chatxai-api/v1', '/coupons/create', [
                'methods'  => 'GET',
                'callback' => [$this, 'createCoupon'],
            ]);

            // Plugin configure
            register_rest_route('chatxai-api/v1', '/plugin/callback', [
                'methods'  => 'GET',
                'callback' => [$this, 'configurePlugin'],
            ]);
        });

        remove_action('rest_api_init', 'create_initial_rest_routes', 0);

        add_filter('rest_route_data', function ($routes) {
            return [];
        });
    }

    // Create posts api
    public function getUserBySessionKey($request)
    {
        if (!$this->checkRequest()) {
            return $this->sendResponseError('Invalid request. Please check the documentation!');
        }

        $session_key = $request->get_param('session_key');

        $users = get_users([
            'meta_key'   => 'chatxai_visitor_key',
            'meta_value' => $session_key,
        ]);

        if (count($users)) {
            $this->success = true;
            $user          = $users[0];

            return $this->sendResponseData([
                'user' => [
                    'id'         => $user->ID,
                    'first_name' => get_user_meta($user->ID, 'first_name', true),
                    'last_name'  => get_user_meta($user->ID, 'last_name', true),
                    'email'      => $user->user_email,
                ],
            ]);
        }

        return $this->sendResponseError('No users found with this token');
    }

    // Create posts api
    public function getUserById($request)
    {
        if (!$this->checkRequest()) {
            return $this->sendResponseError('Invalid request. Please check the documentation!');
        }

        $user_id = $request->get_param('id');
        $user    = get_user_by('id', $user_id);

        if ($user) {
            $this->success = true;
            return $this->sendResponseData([
                'user' => [
                    'id'         => $user->ID,
                    'first_name' => get_user_meta($user->ID, 'first_name', true),
                    'last_name'  => get_user_meta($user->ID, 'last_name', true),
                    'email'      => $user->user_email,
                ],
            ]);
        }

        return $this->sendResponseError('No users found with this token');
    }

    // Create posts api
    public function getProductByID($request)
    {
        if (!$this->checkRequest()) {
            return $this->sendResponseError('Invalid request. Please check the documentation!');
        }

        $product_id = $request->get_param('id');

        $product = WC()->product_factory->get_product($product_id);

        $image = wp_get_attachment_image_src(get_post_thumbnail_id($product->post->ID), 'full');

        if ($product) {
            $this->success = true;
            return $this->sendResponseData([
                'product' => [
                    'name'  => $product->post->post_title,
                    'url'   => get_permalink($product->post->ID),
                    'image' => is_array($image) ? $image[0] : null,
                ],
            ]);
        }

        return $this->sendResponseError('Product not found');
    }

    public function createCoupon($request)
    {

        if (!$this->checkRequest()) {
            return $this->sendResponseError('Invalid request. Please check the documentation!');
        }

        $coupon_code   = isset($_REQUEST['code']) ? $_REQUEST['code'] : 'UNIQUECODE'; // Code
        $description   = isset($_REQUEST['description']) ? $_REQUEST['description'] : 'Created by ChatX'; // Description
        $amount        = isset($_REQUEST['amount']) ? $_REQUEST['amount'] : '1'; // Amount
        $discount_type = isset($_REQUEST['discount_type']) ? $_REQUEST['discount_type'] : 'fixed_cart'; // Type: fixed_cart, percent, fixed_product, percent_product

        $usage_limit    = isset($_REQUEST['usage_limit']) ? $_REQUEST['usage_limit'] : '';
        $expiry_date    = isset($_REQUEST['expiry_date']) ? $_REQUEST['expiry_date'] : '';
        $individual_use = isset($_REQUEST['individual_use']) ? $_REQUEST['individual_use'] : 'false';

        $product_ids = isset($_REQUEST['product_ids']) ? $_REQUEST['product_ids'] : [];

        $coupon_exists = get_page_by_title($coupon_code, OBJECT, 'shop_coupon');

        if ($coupon_exists) {
            return $this->sendResponseError('Coupon already exists');
        }

        $coupon = [
            'post_title'   => $coupon_code,
            'post_content' => '',
            'post_status'  => 'publish',
            'post_author'  => 1,
            'post_type'    => 'shop_coupon',
        ];

        $new_coupon_id = wp_insert_post($coupon);

        // Add meta
        update_post_meta($new_coupon_id, 'discount_type', $discount_type);
        update_post_meta($new_coupon_id, 'coupon_amount', $amount);
        update_post_meta($new_coupon_id, 'individual_use', $individual_use == 'true' ? 'yes' : 'no');

        update_post_meta($new_coupon_id, 'product_ids', implode(',', $product_ids));
        update_post_meta($new_coupon_id, 'exclude_product_ids', '');

        update_post_meta($new_coupon_id, 'usage_limit', $usage_limit);
        update_post_meta($new_coupon_id, 'expiry_date', $expiry_date);

        update_post_meta($new_coupon_id, 'apply_before_tax', 'yes');
        update_post_meta($new_coupon_id, 'free_shipping', 'no');

        $this->success = true;

        return $this->sendResponseData([
            'id'   => $new_coupon_id,
            'code' => $coupon_code,
        ]);
    }

    public function createNotification($request)
    {
        if (!$this->checkRequest()) {
            return $this->sendResponseError('Invalid request. Please check the documentation!');
        }

        $title   = isset($_REQUEST['title']) ? $_REQUEST['title'] : null;
        $message = isset($_REQUEST['message']) ? $_REQUEST['message'] : 'By ChatX.ai';
        $type    = isset($_REQUEST['type']) ? $_REQUEST['type'] : '1';

        $notification_created = true;
        // $title -> String
        // $message -> String
        // $type -> String: info|success|warning|error

        Notices::add($type, $title, $message, true, true);

        if ($notification_created) {
            return $this->sendResponseData();
        } else {
            return $this->sendResponseError();
        }
    }

    public function changeAccountStatus($request)
    {
        if (!$this->checkRequest()) {
            return $this->sendResponseError('Invalid request. Please check the documentation!');
        }

        if (isset($_REQUEST['status'])) {
            Chatx_Ai_Options::update('chatxai_account_status', $_REQUEST['status']);
            if($_REQUEST['status'] == 'active'){
                Notices::removeAll('error');
            }
            return $this->sendResponseData();
        }

        return $this->sendResponseError();
    }

    public function getCronsData($request)
    {
        if (!$this->checkRequest()) {
            return $this->sendResponseError('Invalid request. Please check the documentation!');
        }

        return $this->sendResponseData(cx_get_crons_data());
    }

    public function rescheduleCrons($request)
    {
        if (!$this->checkRequest()) {
            return $this->sendResponseError('Invalid request. Please check the documentation!');
        }

        $cronsData = ['before' => cx_get_crons_data()];

        wp_clear_scheduled_hook('chatxai_send_carts_worker');
        wp_clear_scheduled_hook('chatxai_daily_worker');

        if (!wp_next_scheduled('chatxai_send_carts_worker')) {
            wp_schedule_event(time(), 'chatxai_send_worker_interval', 'chatxai_send_carts_worker');
        }

        if (!wp_next_scheduled('chatxai_daily_worker')) {
            wp_schedule_event(time(), 'daily', 'chatxai_daily_worker');
        }

        $cronsData['after'] = cx_get_crons_data();

        return $this->sendResponseData($cronsData);
    }

    public function setCheckboxTemplates($request)
    {
        if (!$this->checkRequest()) {
            return $this->sendResponseError('Invalid request. Please check the documentation!');
        }

        $templates = isset($_REQUEST['templates']) ? json_decode(stripslashes($_REQUEST['templates']), true) : [];

        if ($templates && is_array($templates)) {
            $templatesFiltered = [];
            foreach ($templates as $template) {
                $tempTemplate = [
                    'id'              => isset($template['id']) ? $template['id'] : null,
                    'campaign'        => isset($template['campaign']) ? $template['campaign'] : null,
                    'button-selector' => isset($template['button-selector']) ? $template['button-selector'] : null,
                    'parent-selector' => isset($template['parent-selector']) ? $template['parent-selector'] : null,
                    'type'            => isset($template['type']) ? $template['type'] : null,
                    'bubble'          => isset($template['bubble']) ? $template['bubble'] : null,
                    'customcss'       => isset($template['customcss']) ? $template['customcss'] : null
                ];

                if ($tempTemplate['id'] && $tempTemplate['type'] && $tempTemplate['campaign']) {
                    $templatesFiltered[] = $tempTemplate;
                }
            }

            Chatx_Ai_Options::update('chatxai_checkbox_templates', $templatesFiltered);
            return $this->sendResponseData();
        }

        return $this->sendResponseError();
    }

    public function getWebsiteConfiguration($request)
    {
        if (!$this->checkRequest()) {
            return $this->sendResponseError('Invalid request. Please check the documentation!');
        }

        global $woocommerce;
        $configuration = [
            'options'  => [],
            'site_url' => get_site_url(),
            'home_url' => get_home_url(),
            'cart_url' => $woocommerce->cart->get_cart_url(),
        ];

        $chatxai_options = [
            'chatxai_messenger_app_id',
            'chatxai_fb_page_id',
            'chatxai_origin',
            'chatxai_enable_add_to_cart_tracking',
            'chatxai_enable_add_to_cart_archive_tracking',
            'chatxai_login_checkbox_enable',
            'chatxai_register_checkbox_enable',
            'chatxai_enable_carts_tracking',
            'chatxai_send_time',
            'chatxai_account_status',
            'chatxai_chatbot_enable_discount',
            'chatxai_chatbot_enable_scarcity',
            'chatxai_chatbot_discount_max_percent',
            'chatxai_billing_company',
            'chatxai_billing_details',
            ['chatxai_checkbox_templates', []],
        ];

        foreach ($chatxai_options as $option_key) {
            $option_value      = null;
            $configuration_key = $option_key;

            if (is_array($option_key)) {
                $configuration_key = $option_key[0];
                $option_value      = Chatx_Ai_Options::read($configuration_key, $option_key[1]);
            } else {
                $option_value = Chatx_Ai_Options::read($option_key);
            }

            $configuration['options'][$configuration_key] = $option_value;
        }

        $configuration['version'] = $this->version;

        try {
            global $woocommerce;
            $configuration['woo_version'] = $woocommerce->version;
            $configuration['currency'] = get_woocommerce_currency();
        } catch (Exception $e) {

        }

        return $this->sendResponseData($configuration);
    }

     public function getSalesReports($request)
    {
        if (!$this->checkRequest()) {
            return $this->sendResponseError('Invalid request. Please check the documentation!');
        }

        $request_start_date = isset($_REQUEST['start_date']) ? $_REQUEST['start_date'] : null;
        $request_end_date = isset($_REQUEST['end_date']) ? $_REQUEST['end_date'] : null;

        try {
            $startDate = new DateTime($request_start_date);
            if(!$request_start_date){
                $startDate->sub(new DateInterval('P7D'));
            }

            $endDate = new DateTime($request_end_date);

            $data = Chatx_Ai_Reports::sales([
                'data' => [
                    '_order_total' => [
                        'type'     => 'meta',
                        'function' => 'SUM',
                        'name'     => 'amount'
                    ],
                    'post_date' => [
                        'type'     => 'post_data',
                        'function' => '',
                        'name'     => 'date'
                    ]
                ],
                'where' => [
                    [
                        'key'      => 'post_date',
                        'value'    => $startDate->format('Y-m-d H:i:s'),
                        'operator' => '>='
                    ],
                    [
                        'key'      => 'post_date',
                        'value'    => $endDate->format('Y-m-d H:i:s'),
                        'operator' => '<=',
                    ]
                ],
                'group_by'     => 'YEAR(posts.post_date), MONTH(posts.post_date), DAY(posts.post_date)',
                'query_type'   => 'get_results',
                'filter_range' => false
            ]);

            return $this->sendResponseData($data);

        } catch (Exception $e) {
            return $this->sendResponseError($e->getMessage());
        }
    }

    public function configurePlugin($request)
    {
        $api_key    = null;
        $api_token  = null;
        $fb_app_id  = null;
        $fb_page_id = null;

        $verify_key = $request->get_param('verify_key');

        try {
            $verify   = file_get_contents(CHATX_BAESURL . '/plugin/verify?verify=' . $verify_key);
            $response = json_decode($verify);

            if (isset($response->success) && $response->success === true) {
                $api_key    = isset($response->api_key) ? $response->api_key : null;
                $api_token  = isset($response->api_token) ? $response->api_token : null;
                $fb_app_id  = isset($response->fb_app_id) ? $response->fb_app_id : null;
                $fb_page_id = isset($response->fb_page_id) ? $response->fb_page_id : null;

                if ($api_key && $api_token && $fb_app_id && $fb_page_id) {
                    $this->success = true;
                }
            }

        } catch (Exception $e) {
            $this->success = false;
        }

        if (!$this->success) {
            Notices::add('error', 'ChatX configuration error', 'Invalid ChatX response... Error on callback validation.', false, false, true);
            wp_redirect(admin_url('admin.php?page=chatxai-dashboard'));
            exit;
        }

        // Default settings
        $chatxai_options = [
            'chatxai_api_token'                           => $api_token,
            'chatxai_api_key'                             => $api_key,
            'chatxai_messenger_app_id'                    => $fb_app_id,
            'chatxai_fb_page_id'                          => $fb_page_id,
            'chatxai_origin'                              => get_site_url(),
            'chatxai_enable_add_to_cart_tracking'         => 'yes',
            'chatxai_enable_add_to_cart_archive_tracking' => 'yes',
            'chatxai_login_checkbox_enable'               => 'yes',
            'chatxai_register_checkbox_enable'            => 'yes',
            'chatxai_enable_carts_tracking'               => 'yes',
            'chatxai_send_time'                           => 6,
            'chatxai_account_status'                      => 'active',
            'chatxai_chatbot_enable_discount'             => 'no',
            'chatxai_chatbot_enable_scarcity'             => 'no',
            'chatxai_chatbot_discount_max_percent'        => 0,
        ];

        foreach ($chatxai_options as $option => $value) {
            Chatx_Ai_Options::update($option, $value);
        }

        Notices::add('success', 'Congratulations!', 'ChatX was successfully configured on your shop. Let\'s recover some abandoned carts!', false, false, true);
        wp_redirect(admin_url('admin.php?page=chatxai-dashboard'));
        exit;
    }

    // Check if API key exists and is valid
    public function checkRequest()
    {
        if (!isset($_REQUEST['api_key']) || $_REQUEST['api_key'] !== $this->apiKey) {
            return false;
        }

        if ($_SERVER['REQUEST_METHOD'] == "PUT") {
            parse_str(file_get_contents("php://input"), $_POST);
        }

        return true;
    }

    // Send response
    public function sendResponse($echo = false)
    {
        $responseData = [
            'code'    => "ok",
            'success' => $this->success,
            'data'    => $this->data,
            'message' => $this->message,
        ];

        if (!$echo) {
            return $responseData;
        }

        wp_send_json($responseData);
    }

    // Send response with error message
    public function sendResponseError($error = "With error", $data = [], $echo = false)
    {
        $this->message = $error;
        $this->success = false;
        $this->data    = $data;
        return $this->sendResponse($echo);
    }

    // Send response with data and success
    public function sendResponseData($data = [], $msg = "Success", $echo = false)
    {
        $this->message = $msg;
        $this->data    = $data;
        $this->success = true;
        return $this->sendResponse($echo);
    }
}
