<?php

namespace CXModule\AbandonedCarts;

use WordPress\ORM\AbandonedCart;
use WordPress\ORM\RestoredCart;
use WordPress\ORM\Guest;

/**
 * @class Abandoned_Cart_Manager
 */
class Abandoned_Cart_Manager
{

    /** @var bool - used when restoring carts so that we don't fire unnecessary db queries */
    public $_prevent_store_cart = false;

    /**
     * Constructor
     */
    public function __construct()
    {
        add_action('chatxai_daily_worker', [$this, 'clean_stored_carts']);
        add_action('woocommerce_cart_updated', [$this, 'maybe_store_cart'], 100);
        add_action('woocommerce_cart_emptied', [$this, 'cart_emptied']);
        add_action('woocommerce_checkout_order_processed', [$this, 'empty_after_order_created']);
    }

    /**
     * Logic to determine whether we should save the cart on certain hooks
     */
    public function maybe_store_cart()
    {

        if ( did_action( 'wp_logout' ) ) return; // don't clear the cart after logout
        //if ( is_admin() ) return;

        if ($this->_prevent_store_cart) {
            return;
        }

        if ($user_id = ChatXACT()->session_tracker->get_detected_user_id()) {
            $this->store_user_cart($user_id);
        } elseif ($guest = ChatXACT()->session_tracker->get_current_guest()) {
            $this->store_guest_cart($guest);
            $guest->setLastActive(new \DateTime());
            $guest->save();
        }
    }

    /**
     * Attempts to update or insert carts for guests
     *
     * @param Guest $guest
     * @return bool
     */
    public function store_guest_cart($guest)
    {
        $is_empty_cart = !count(WC()->cart->get_cart());

        if (!$guest) {
            return false;
        }

        $cart = AbandonedCart::find_one_by('guest_id', $guest->id);

        if ($cart) {
            if ($is_empty_cart) {
                $cart->delete();
                return false;
            }

            $cart->sync();
            return true;
        }

        // We need to create the cart
        // cart is empty
        if ($is_empty_cart) {
            return false;
        }

        // create new cart
        $cart = new AbandonedCart;
        $cart->setCreated(new \DateTime());
        $cart->guest_id = $guest->id;
        $cart->setToken();
        $cart->sync();
        return true;
    }

    /**
     * Attempts to store cart for a registered user whether they are logged in or not
     *
     * @param bool $user_id
     * @return bool
     */
    public function store_user_cart($user_id = false)
    {
        if (!$user_id) {
            if (!$user_id = ChatXACT()->session_tracker->get_detected_user_id()) {
                return false;
            }
        }

        $is_empty_cart = !count(WC()->cart->get_cart());

        $existing_cart = AbandonedCart::find_one_by('user_id', $user_id);

        if ($existing_cart) {
            if ($is_empty_cart) {
                $existing_cart->delete();
                return false;
            }

            $existing_cart->sync();
            return true;
        }

        if ($is_empty_cart) {
            return false;
        }

        // create a new stored cart for the user
        $cart = new AbandonedCart;
        $cart->setCreated(new \DateTime());
        $cart->user_id = $user_id;
        $cart->setToken();
        $cart->sync();
        return true;
    }

    /**
     * This event will fire when an order is placed and the cart is emptied NOT when a user empties their cart.
     */
    public function cart_emptied()
    {
        // don't clear the cart after logout
        if ($this->_prevent_store_cart || did_action('wp_logout')) {
            return;
        }

        $guest   = ChatXACT()->session_tracker->get_current_guest();
        $user_id = ChatXACT()->session_tracker->get_detected_user_id();

        if ($user_id) {
            $cart = AbandonedCart::find_one_by('user_id', $user_id);
            if ($cart) {
                $cart->delete();
            }
        }

        if ($guest) {
            $cart = AbandonedCart::find_one_by('guest_id', $guest->id);
            if ($cart) {
                $cart->delete();
            }
        }
    }

    public function create_restore_cart($cart, $order_id){
        $restored_cart = RestoredCart::find_one_by('token', $cart->token);
        if(!$restored_cart){
            $restored_cart = new RestoredCart;
        }

        $restored_cart->setCreated(new \DateTime());
        $restored_cart->user_id = $cart->user_id;
        $restored_cart->guest_id = $cart->guest_id;
        $restored_cart->order_id = $order_id;
        $restored_cart->token = $cart->token;
        $restored_cart->items = $cart->items;
        $restored_cart->sync();

        // Remove abandoned cart
        $cart->delete();

        ChatXACT()->send_restored_cart($restored_cart);

        return $restore_cart;
    }

    // Restore cart only if the cart was sent to chatx
    public function restore_only_chatx_cart($cart, $order_id){
        if($cart->sent){
            $this->create_restore_cart($cart, $order_id);
        }
        else{
            $cart->delete();
        }
    }

    /**
     * Ensure the stored abandoned cart is removed when an order is created
     *
     * Clears even if payment has not gone through
     *
     * @param $order_id
     */
    public function empty_after_order_created($order_id)
    {
        $order   = wc_get_order($order_id);
        $user_id = $order->get_user_id();

        if ($user_id) {
            if ($cart = AbandonedCart::find_one_by('user_id', $user_id)){
                $this->restore_only_chatx_cart($cart, $order_id);
            }
        }

        // order placed by a guest or by a guest that signed up at checkout
        $guest = Guest::find_one_by('email', strtolower(\CXLibrary\CXOrder::get_billing_email($order)));
        if($guest){
            if ($cart = AbandonedCart::find_one_by('guest_id', $guest->id)){
                $this->restore_only_chatx_cart($cart, $order_id);
            }
        }

        // Delete cart by guest cookie key
        if ($guest = ChatXACT()->session_tracker->get_current_guest()) {
            if ($cart = AbandonedCart::find_one_by('guest_id', $guest->id)){
                $this->restore_only_chatx_cart($cart, $order_id);
            }
        }
    }

    /**
     * Restores a cart into the current session
     * @param bool $cart_token
     */
    public function restore_cart($cart_token)
    {
        // block cart storage hooks
        $this->_prevent_store_cart = true;

        if (!$cart_token) {
            return;
        }

        $cart = AbandonedCart::find_one_by('token', $cart_token);

        if (!$cart || !is_array($cart->items)) {
            return;
        }

        $notices_backup            = wc_get_notices();

        // Empty current cart
        WC()->cart->empty_cart();

        foreach ($cart->items as $item_key => $item) {
            WC()->cart->add_to_cart($item['product_id'], $item['quantity'], $item['variation_id'], $item['variation']);
        }

        // restore coupons
        foreach ($cart->coupons as $coupon_code => $coupon_data) {
            if (!WC()->cart->has_discount($coupon_code)) {
                WC()->cart->add_discount($coupon_code);
            }
        }

        // clear show notices for added coupons or products
        WC()->session->set('wc_notices', $notices_backup);

        // restore user session
        $this->restore_session($cart);

        ChatXACT()->send_clicked_cart($cart);

        // unblock cart storing and store the restored cart
        $this->_prevent_store_cart = false;

        wp_redirect(add_query_arg([
            'cx-cart-restored' => 1,
        ], wc_get_page_permalink('cart')));

        exit;
    }

    /**
     * Restore user session
     */
    public function restore_session($cart = null){
        $session_key = $this->get_cart_session_key($cart);

        // If cart don't have session_key ... STOP
        if(!$session_key){
            return;
        }

        // Remove my current cart if not the same with $cart
        $this->remove_my_cart_if_not_same($cart);

        // Set new seession key and fetch the right guest
        ChatXACT()->session_tracker->restore_session($session_key);

        // If i'm logged in, update abandoned cart with my id
        if($user_id = ChatXACT()->session_tracker->get_detected_user_id()){
            $cart->user_id = $user_id;
        }
        else if($guest = ChatXACT()->session_tracker->get_current_guest()){
            $cart->guest_id = $guest->primary_key();
        }

        // Save cart
        $cart->save();
    }

    /**
     * Remove current guest or user abandoned cart if not the same
    */
    public function remove_my_cart_if_not_same($cart){
        $user_id = ChatXACT()->session_tracker->get_detected_user_id();
        $guest = ChatXACT()->session_tracker->get_current_guest();
        $guest_id = $guest && $guest->primary_key() ? (int)$guest->primary_key() : null;

        if($user_id && (int)$user_id !== (int)$cart->user_id){
            $current_cart = AbandonedCart::find_one_by('user_id', $user_id);
            if ($current_cart && $current_cart->token !== $cart->token){
                $current_cart->delete();
            }
        }
        if($guest_id && guest_id !== (int)$cart->guest_id){
            $current_cart = AbandonedCart::find_one_by('guest_id', $guest->primary_key());
            if ($current_cart && $current_cart->token !== $cart->token){
                $current_cart->delete();
            }
        }
    }

     /**
     * Get guest/user tracking_key of cart
    */
    public function get_cart_session_key($cart){
        if (!$cart) {
            return false;
        }

        if ($cart->user_id) {
            $user = get_user_by('id', $cart->user_id);
            $session_key = $user ? get_user_meta($user->ID, 'chatxai_visitor_key', true) : null;
        }
        else if ($cart->guest_id) {
            $guest = Guest::find_one($cart->guest_id);
            $session_key = $guest ? $guest->tracking_key : null;
        }

        return $session_key;
    }

    /**
     * Delete stored carts older than 10 days
     */
    public function clean_stored_carts()
    {
        $delay_date = new \DateTime(); // UTC
        $delay_date->modify("-10 days");

        $old_abandoned_carts = AbandonedCart::query()
            ->where_lt('last_modified', $delay_date->format('Y-m-d H:i:s'))
            ->find();

        foreach ($old_abandoned_carts as $cart) {
            $cart->delete();
        }

        $this->clean_older_coupons();
    }

    /**
     * Delete coupons older than 2 days
     */
    public function clean_older_coupons()
    {
        $args = array(
            'posts_per_page' => -1,
            'post_type' => 'shop_coupon',
            'post_status' => 'publish',
            'date_query' => array(
                'before' => date('Y-m-d', strtotime('-2 days'))
            ),
            'meta_query' => [
                [
                    'key' => 'usage_count',
                    'compare' => 'NOT EXISTS'
                ]
            ],
            's' => 'chatx'
        );

        $coupons = get_posts( $args );

        if (!empty($coupons)) {
            $unable_to_delete = array();
            foreach( $coupons as $coupon ) {
                if (false === wp_delete_post($coupon->ID)) {
                    $unable_to_delete[$coupon->ID] = $coupon->post_title;
                }
            }
        }
    }
}
