<?php

namespace CXModule\AbandonedCarts;

use Chatx_Ai_Options;
use CXLibrary\Clean;
use WordPress\ORM\AbandonedCart;
use WordPress\ORM\Guest;
use WordPress\ORM\RestoredCart;

final class ChatXAC
{

    /** @var ChatXAC\Abandoned_Cart_Manager */
    public $abandoned_cart;

    /** @var ChatXAC\Session_Tracker */
    public $session_tracker;

    /** @var ChatXAC\Admin */
    public $admin;

    /** @var ChatXAC */
    private static $_instance = null;

    private function __construct()
    {
        add_action('chatxai_send_carts_worker', [$this, 'check_abandoned_carts']);
        add_action('woocommerce_init', [$this, 'init'], 20);
    }

    public function init()
    {
        $this->session_tracker = Session_Tracker_Instance();
        $this->abandoned_cart  = new Abandoned_Cart_Manager;

        add_action('wp_loaded', [$this, 'check_for_action_endpoint']);

        if (is_admin()) {
            require_once __DIR__ . '/admin/admin.php';

            require_once __DIR__ . '/admin/controllers/abstract.php';
            require_once __DIR__ . '/admin/controllers/carts.php';
            require_once __DIR__ . '/admin/controllers/restored-carts.php';

            Admin::init();
        }
    }

    public function install()
    {
        Guest::create_table();
        AbandonedCart::create_table();
        RestoredCart::create_table();

        if (!wp_next_scheduled('chatxai_send_carts_worker')) {
            wp_schedule_event(time(), 'chatxai_send_worker_interval', 'chatxai_send_carts_worker');
        }

        if (!wp_next_scheduled('chatxai_daily_worker')) {
            wp_schedule_event(time(), 'daily', 'chatxai_daily_worker');
        }

        $this->set_plugin_state(true);
    }

    public function uninstall()
    {
        wp_clear_scheduled_hook('chatxai_send_carts_worker');
        wp_clear_scheduled_hook('chatxai_daily_worker');

        $this->set_plugin_state(false);
    }

    public function set_plugin_state($state = false){
        try {
            $this->do_chatx_post_request('/plugin/state', ['state' => $state]);
        } catch (Exception $e) {

        }
    }

    /**
     * Action endpoints
     */
    public static function check_for_action_endpoint()
    {
        if (isset($_GET['cx-action']) && !empty($_GET['cx-action'])) {
            if (is_admin()) {
                ChatXACT()->handle_admin_action(sanitize_key(cx_request('cx-action')));
            } else {
                ChatXACT()->handle_action(sanitize_key(cx_request('cx-action')));
            }
        }
    }

    public static function handle_action($action)
    {
        switch ($action) {
            case 'restore-cart':
                $token = Clean::string(cx_request('token'));
                ChatXACT()->abandoned_cart->restore_cart($token);
                break;
        }
    }

    public static function handle_admin_action($action)
    {
        $self = $this;
        switch ($action) {
            case 'send-dev-cart':
                $cart_ids = Clean::ids(cx_request('cart_id'));

                foreach ($cart_ids as $cart_id) {
                    $cart = AbandonedCart::find_one($cart_id);

                    try {
                        if (cx_request('order') == '1') {
                            $cart->order_id = 9999;
                            $result         = ChatXACT()->send_restored_cart($cart);
                        } else {
                            $result = ChatXACT()->send_abandoned_cart($cart);
                        }

                    } catch (Exception $e) {
                        $result = false;
                    }

                    echo "Cart " . $cart_id . " -> Sending -> ";
                    if ($cart) {
                        echo $result ? "OK" : "FAILED";
                    } else {
                        echo "NOT FOUND";
                    }
                }
                exit;
                break;
        }
    }

    public function check_abandoned_carts()
    {
        $delay_date = new \DateTime(); // UTC
        $delay_date->modify("-" . Chatx_Ai_Options::read('chatxai_send_time') . " hours");

        $abandoned_carts = AbandonedCart::query()
            ->where_lt('last_modified', $delay_date->format('Y-m-d H:i:s'))
            ->where_null('sent')
            ->find();

        foreach ($abandoned_carts as $cart) {
            $was_sent = ChatXACT()->send_abandoned_cart($cart);
        }
    }

    public function get_ip()
    {
        if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
            $ip = $_SERVER['HTTP_CLIENT_IP'];
        } elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
            $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
        } else {
            $ip = $_SERVER['REMOTE_ADDR'];
        }

        return $ip;
    }

    public static function instance()
    {
        if (is_null(self::$_instance)) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    public function prepare_cart_data($cart)
    {
        if (!$cart) {
            return false;
        }

        $user  = null;
        $guest = null;

        if ($cart->user_id) {
            $user = get_user_by('id', $cart->user_id);
        }

        if ($cart->guest_id) {
            $guest = Guest::find_one($cart->guest_id);
        }

        if (!$user && !$guest) {
            return false;
        }

        $data = [
            'cart'        => $cart->getApiFormat()
        ];

        if ($user) {
            $data['user'] = $user->ID;
            $data['session_key'] = get_user_meta($user->ID, 'chatxai_visitor_key', true);
        }

        if ($guest) {
            $data['guest'] = $guest->to_array();
            $data['session_key'] = $guest->tracking_key;
        }

        return $data;
    }

    public function do_chatx_post_request($url, $data)
    {
        $data['at'] = \Chatx_Ai_Options::read('chatxai_api_token');
        $data['ak'] = \Chatx_Ai_Options::read('chatxai_api_key');

        if(!$data['at'] || !$data['ak']){
            return false;
        }

        $options = array(
            'http' => array(
                'header'  => "Content-type: application/x-www-form-urlencoded\r\n",
                'method'  => 'POST',
                'content' => http_build_query($data),
            ),
        );
        $url     = CHATX_BAESURL . '/' . $url;
        $context = stream_context_create($options);
        $result  = file_get_contents($url, false, $context);

        if ($result === false) {
            return false;
        }

        return json_decode($result);
    }

    public function convert_guest_to_user($guest, $user)
    {
        $guest_id = $guest->primary_key();

        $abandoned_carts_old = AbandonedCart::query()->where('user_id', $user->ID)->where_not('guest_id', $guest_id)->find();
        foreach ($abandoned_carts_old as $cart) {
            $cart->delete();
        }

        $abandoned_carts = AbandonedCart::query()->where('guest_id', $guest_id)->find();
        foreach ($abandoned_carts as $cart) {
            $cart->guest_id = null;
            $cart->user_id  = $user->ID;
            $cart->save();
        }

        $restored_carts = RestoredCart::query()->where('guest_id', $guest_id)->find();
        foreach ($restored_carts as $cart) {
            $cart->guest_id = null;
            $cart->user_id  = $user->ID;
            $cart->save();
        }

        $guest->delete();
    }

    public function send_abandoned_cart($cart)
    {
        $data = $this->prepare_cart_data($cart);
        if (!$data) {
            return false;
        }

        $result = $this->do_chatx_post_request('plugin/abandoned', $data);
        if (!$result || !$result->success) {
            $cart->incrementTries();
            return false;
        }

        $now        = new \DateTime();
        $cart->sent = $now->format('Y-m-d H:i:s');
        $cart->save();

        return true;
    }

    public function send_clicked_cart($cart)
    {
        $data = $this->prepare_cart_data($cart);
        if (!$data) {
            return false;
        }

        $result = $this->do_chatx_post_request('plugin/clicked', $data);
        return $result && $result->success;
    }

    public function send_restored_cart($cart)
    {
        $data = $this->prepare_cart_data($cart);
        if (!$data) {
            return false;
        }

        $result = $this->do_chatx_post_request('plugin/restored', $data);
        return $result && $result->success;
    }

    public function get_current_abandoned_cart(){
        $user_id = $this->session_tracker->get_detected_user_id();
        $guest = $this->session_tracker->get_current_guest();
        $guest_id = $guest && $guest->primary_key() ? (int)$guest->primary_key() : null;
        $current_cart = null;

        if($user_id){
            $current_cart = AbandonedCart::find_one_by('user_id', $user_id);
        }
        else if($guest_id){
            $current_cart = AbandonedCart::find_one_by('guest_id', $guest->primary_key());
        }

        return $current_cart;
    }

    public function get_cart($id = null, $type = "abandoned"){
        if(!$id) return false;

        if(!$type || $type == "abandoned"){
            return AbandonedCart::find_one_by('id', $id);
        }

        return RestoredCart::find_one_by('id', $id);
    }
}
