<?php

namespace CXModule;

use WordPress\ORM\Guest;
use CXLibrary\Clean;

class Session_Tracker
{

    /** @var int (days) */
    private $tracking_cookie_expiry;

    /** cookie name */
    private $tracking_key_cookie_name;

    /** @var string */
    private $tracking_key;

    /** @var int */
    private $user_id;

    /** @var Guest|false */
    private $guest;

    /** @var bool */
    private $tracking_key_to_set = false;

    private static $_instance = null;
    /**
     * Construct
     */
    public function __construct()
    {

        $this->tracking_key_cookie_name = apply_filters('chatxai/session_tracker/cookie_name', 'wp_chatxai_visitor_' . COOKIEHASH);
        $this->tracking_cookie_expiry   = apply_filters('chatxai_visitor_tracking_cookie_expiry', 730); // 2 years

        add_action('wp', [$this, 'maybe_set_session_cookies'], 99);
        add_action('init', [$this, 'plugin_init'], 99);
        add_action('shutdown', [$this, 'maybe_set_session_cookies'], 0);

        //set_logged_in_cookie
        add_action('set_logged_in_cookie', [$this, 'user_login'], 10, 4);

        add_action('comment_post', [$this, 'capture_from_comment'], 10, 2);
        add_action('chatxai_capture_guest_email', [$this, 'store_guest']); // for third-party
    }

    public static function instance()
    {
        if (is_null(self::$_instance)) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    public function plugin_init(){
        $this->get_tracking_key();
    }

    /**
     * @param bool|int $user_id
     * @param bool $force_refresh
     */
    public function init_session($user_id = false, $force_refresh = false)
    {
        if (did_action('chatxai_session_initiated') && !$force_refresh) {
            return;
        }

        $cookie_key = isset($_COOKIE[$this->tracking_key_cookie_name]) ? Clean::string($_COOKIE[$this->tracking_key_cookie_name]) : false;

        if (!$user_id) {
            $user_id = get_current_user_id();
        }

        if ($user_id) {

            $database_key = get_user_meta($user_id, 'chatxai_visitor_key', true);

            // is tracking cookie set?
            if ($cookie_key) {

                if ($database_key) {

                    if ($database_key != $cookie_key) {
                        // if a database key exists but is different from the cookie key, update the cookie key
                        $this->tracking_key_to_set = $database_key;
                    } else {
                        // cookie and db match
                        $this->tracking_key = $database_key;
                    }
                } else {
                    // cookie key exists but is not in db yet

                    // before storing remove this key from any other users
                    // ensures anyone with multiple user accounts always has the session key attached to their most recent login
                    delete_metadata('user', $user_id, 'chatxai_visitor_key', $cookie_key, true);

                    update_user_meta($user_id, 'chatxai_visitor_key', $cookie_key);
                    $this->tracking_key = $cookie_key;
                }
            } elseif ($database_key) {
                // no cookie key set but there is a database key so lets use that
                $this->tracking_key_to_set = $database_key;
            } else {
                // no cookie key or stored key, lets generate a new one
                $this->tracking_key_to_set = $this->generate_key();
                update_user_meta($user_id, 'chatxai_visitor_key', $this->tracking_key_to_set);
            }
        } else {
            if ($cookie_key) {
                // guest has cookie key so use it
                $this->tracking_key = $cookie_key;
            } else {
                // guest has no cookie so generate one
                $this->tracking_key_to_set = $this->generate_key();
            }
        }

        if (empty($_COOKIE['wp_chatxai_session_started'])) {
            $this->new_session_initiated();
        }

        do_action('chatxai_session_initiated');
    }

    /**
     * New browser session initiated.
     */
    public function new_session_initiated()
    {

        // update guest ip and active date for the new session
        if ($guest = $this->get_current_guest()) {
            $guest->ip = ChatXACT()->get_ip();
            $guest->setLastActive(new \DateTime());
            $guest->save();
        }

        do_action('chatxai_new_session_initiated');
    }

    /**
     * Clear the stored guest before their cookie key is updated.
     * Store the cart upon login
     *
     * @param $logged_in_cookie
     * @param $expire
     * @param $expiration
     * @param $user_id
     */
    public function user_login($logged_in_cookie, $expire, $expiration, $user_id)
    {
        $user = get_user_by('id', $user_id);

        if (!$user || !$user->user_email) {
            return;
        }

        // find a matching guest by email
        $guest = Guest::find_one_by('email', Clean::email($user->user_email));

        if (!$guest) {
            // find matching guest by tracking key
            if ($tracking_key = ChatXACT()->session_tracker->get_tracking_key()) {
                $guest = Guest::find_one_by('tracking_key', $tracking_key);
            }
        }

        // if a guest matches the logging in user convert the guest to a user
        if ($guest) {
            ChatXACT()->convert_guest_to_user($guest, $user);
        }

        // init the session after login so new tracking key gets set
        $this->init_session($user->ID, true);
    }

    /**
     * Attempt to set session tracking cookies
     */
    public function maybe_set_session_cookies()
    {
        if (!$this->is_possible_to_set_cookies()) {
            return;
        }

        if ($this->tracking_key_to_set) {
            $this->tracking_key        = $this->tracking_key_to_set;
            $this->tracking_key_to_set = false;

            wc_setcookie($this->tracking_key_cookie_name, $this->tracking_key, time() + DAY_IN_SECONDS * $this->tracking_cookie_expiry);
        }

        if ($this->tracking_key) {
            wc_setcookie('wp_chatxai_session_started', 1);
        }
    }

    /**
     * @return bool
     */
    public function is_possible_to_set_cookies()
    {
        return !headers_sent();
    }

    /**
     * Returns false if not id is set
     *
     * @return string|false
     */
    public function get_tracking_key()
    {
        $this->init_session();

        if ($this->tracking_key) {
            return $this->tracking_key;
        }

        if ($this->tracking_key_to_set && $this->is_possible_to_set_cookies()) {
            return $this->tracking_key_to_set;
        }

        return false;
    }

    /**
     * Restore a session by session_key
     *
     * @return string|false
     */
    public function restore_session($session_key)
    {
        if($session_key == $this->get_tracking_key()){
            return;
        }

        $guest = $this->get_current_guest();
        if($guest){
            $guest->delete();
        }

        $this->tracking_key_to_set = $session_key;
        $this->maybe_set_session_cookies();

        $this->guest = null;
        $this->get_current_guest();
    }

    /**
     * @return string
     */
    public function generate_key()
    {
        return cx_generate_key(32);
    }

    /**
     * Detect the user id based from their visitor id
     *
     * Returns false if no use is detected.
     *
     * @return int
     */
    public function get_detected_user_id()
    {
        if (is_user_logged_in()) {
            return (int)get_current_user_id();
        }

        if (isset($this->user_id)) {
            return (int)$this->user_id;
        }

        $this->user_id = 0;
        $this->init_session();

        // only search for existing tracking keys not newly set ones
        if ($this->tracking_key) {

            $user_query = new \WP_User_Query([
                'meta_query' => [
                    [
                        'key'   => 'chatxai_visitor_key',
                        'value' => $this->tracking_key,
                    ],
                ],
                'fields'     => 'ids',
            ]);

            $results = $user_query->get_results();

            if (!empty($results)) {
                $this->user_id = (int)$results[0];
            }
        }

        return (int)$this->user_id;
    }

    /**
     * @return Guest|bool
     */
    public function get_current_guest()
    {
        if (!isset($this->guest)) {

            $this->guest  = false;
            $tracking_key = $this->get_tracking_key();

            if ($tracking_key && !is_user_logged_in()) {

                $guest = Guest::find_one_by('tracking_key', $tracking_key);

                if ($guest) {
                    $this->guest = $guest;
                }
                else{
                	$this->guest = $this->store_basic_guest();
                }
            }
        }

        return $this->guest;
    }

    public function store_basic_guest(){
		if (!$tracking_key = $this->get_tracking_key()) {
            return false;
        }

		$guest               = new Guest;
        $guest->email        = '';
        $guest->tracking_key = $tracking_key;
        $guest->setCreated(new \DateTime());
        $guest->ip = ChatXACT()->get_ip();

		return $guest->save() ? $guest : null;
    }

    /**
     * Store the guest against the current session key
     * Also can be used to update the current guest's email
     *
     * @param $email
     * @param string|bool $language
     * @param string|bool $location - location the guest was captured
     * @return Guest|false
     */
    public function store_guest($email, $language = false, $location = false)
    {
    	// if there is no tracking key or no ability to set one there is no point storing the guest
        if (!$tracking_key = $this->get_tracking_key()) {
            return false;
        }

        if (!is_email($email) || is_user_logged_in()) {
            return false;
        }

        $email = Clean::email($email);
        $guest = $this->get_current_guest();

        if ($guest) {
            if ($guest->email !== $email) {
                $guest->email = $email;
            }

            $guest->save();

            return $guest;
        } else {
            // creating a new guest but first check if the guest email is already stored
            $stored_new_guest = false;

            $guest = Guest::find_one_by('email', $email);

            if ($guest) {
                // guest email is stored already so update the current session tracking key to match
                $this->tracking_key_to_set = $guest->tracking_key;
            } else {
                $guest               = new Guest;
                $guest->email        = $email;
                $guest->tracking_key = $tracking_key;
                $guest->setCreated(new \DateTime());
                $guest->save();

                $stored_new_guest = true;
            }

            $guest->setLastActive(new \DateTime());
            $guest->ip = ChatXACT()->get_ip();

            if ($language) {
                $guest->language = $language;
            }

            $guest->save();

            if ($location) {
                //$guest->add_meta( 'location_captured', $location );
            }

            // save the cart
            ChatXACT()->abandoned_cart->store_guest_cart($guest);

            if ($stored_new_guest) {
                // fire hook after new guest is saved
                do_action('chatxai/session_tracker/new_stored_guest', $guest);
            }

            return $guest;
        }
    }

    /**
     * Store guest info if they place a comment
     * @param $comment_ID
     */
    public function capture_from_comment($comment_ID)
    {

        $comment = get_comment($comment_ID);

        if ($comment && !$comment->user_id) {
            $this->store_guest($comment->comment_author_email);
        }
    }
}
